/*
//@HEADER
// ************************************************************************
//
//               Epetra: Linear Algebra Services Package
//                 Copyright 2011 Sandia Corporation
//
// Under the terms of Contract DE-AC04-94AL85000 with Sandia Corporation,
// the U.S. Government retains certain rights in this software.
//
// Redistribution and use in source and binary forms, with or without
// modification, are permitted provided that the following conditions are
// met:
//
// 1. Redistributions of source code must retain the above copyright
// notice, this list of conditions and the following disclaimer.
//
// 2. Redistributions in binary form must reproduce the above copyright
// notice, this list of conditions and the following disclaimer in the
// documentation and/or other materials provided with the distribution.
//
// 3. Neither the name of the Corporation nor the names of the
// contributors may be used to endorse or promote products derived from
// this software without specific prior written permission.
//
// THIS SOFTWARE IS PROVIDED BY SANDIA CORPORATION "AS IS" AND ANY
// EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
// IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR
// PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL SANDIA CORPORATION OR THE
// CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL,
// EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO,
// PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR
// PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF
// LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING
// NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS
// SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
//
// Questions? Contact Michael A. Heroux (maherou@sandia.gov)
//
// ************************************************************************
//@HEADER
*/

#ifndef EPETRA_GIDTYPEVECTOR_H
#define EPETRA_GIDTYPEVECTOR_H

#include "Epetra_ConfigDefs.h"
#include "Epetra_IntVector.h"
#include "Epetra_LongLongVector.h"

//! Epetra_GIDTypeVector: A class for constructing and using dense "int" and "long long" vectors on a parallel computer.

/*! The Epetra_GIDTypeVector class enables the construction and use of integer
    dense vectors in a distributed memory environment.
*/

//=========================================================================
template<typename int_type>
class Epetra_GIDTypeVector;

#ifndef EPETRA_NO_32BIT_GLOBAL_INDICES

template<>
class Epetra_GIDTypeVector<int>
{
public:
	typedef Epetra_IntVector impl;
};

#endif

#ifndef EPETRA_NO_64BIT_GLOBAL_INDICES

template<>
class Epetra_GIDTypeVector<long long>
{
public:
	typedef Epetra_LongLongVector impl;
};

#endif

#endif /* EPETRA_GIDTYPEVECTOR_H */
